@@ -25,9 +25,6 @@
 #'
 
 as_tabyl <- function(dat, axes = 2, row_var_name = NULL, col_var_name = NULL) {
-  if ("tabyl" %in% class(dat)) {
-    return(dat)
-  }
   if (!axes %in% 1:2) {
     stop("axes must be either 1 or 2")
   }
@@ -41,12 +38,23 @@ as_tabyl <- function(dat, axes = 2, row_var_name = NULL, col_var_name = NULL) {
   }
   
   # assign core attribute and classes
-  attr(dat, "core") <- as.data.frame(dat) # core goes first so dat does not yet have attributes attached to it
-  attr(dat, "tabyl_type") <- dplyr::case_when(
-    axes == 1 ~ "one_way",
-    axes == 2 ~ "two_way"
-  )
-  class(dat) <- c("tabyl", class(dat))
+  if("tabyl" %in% class(dat)){
+    # if already a tabyl, may have totals row.  Safest play is to simply reorder the core rows to match the dat rows
+    attr(dat, "core") <- attr(dat, "core")[order(match(attr(dat, "core")[, 1],
+                                                       dat[, 1])), ]
+    row.names(attr(dat, "core")) <- 1:nrow(attr(dat, "core")) # if they're sorted in the prior step above, this resets
+  } else {
+    attr(dat, "core") <- as.data.frame(dat) # core goes first so dat does not yet have attributes attached to it
+  }
+  
+  attr(dat, "tabyl_type") <- ifelse(
+    !is.null(attr(dat, "tabyl_type")),
+    attr(dat, "tabyl_type"), # if a one_way tabyl has as_tabyl called on it, it should stay a one_way #523
+    dplyr::case_when(
+      axes == 1 ~ "one_way",
+      axes == 2 ~ "two_way"
+    ))
+  class(dat) <- c("tabyl", setdiff(class(dat), "tabyl"))
   
   if (!missing(row_var_name) | !missing(col_var_name)) {
     if (axes != 2) {
