@@ -1,5 +1,3 @@
-context("clean_names")
-
 # Do not run the tests if not all transliterators are available
 
 options(janitor_warn_transliterators=NULL)
@@ -108,14 +106,16 @@ test_that("All scenarios for make_clean_names", {
     make_clean_names("jan 2009 sales"),
     "jan_2009_sales"
   )
-  expect_equal(
+  # warning due to unhandled mu
+  expect_warning(expect_equal(
     make_clean_names("not_first_unicode_µ"),
     "not_first_unicode_m"
-  )
-  expect_equal(
+  ))
+  # warning due to unhandled mu
+  expect_warning(expect_equal(
     make_clean_names("µ_first_unicode"),
     "m_first_unicode"
-  )
+  ))
   expect_equal(
     make_clean_names("a/b"),
     "a_b",
@@ -126,13 +126,13 @@ test_that("All scenarios for make_clean_names", {
     "a_per_b",
     info="Custom replacement"
   )
-  
+
   expect_equal(
     make_clean_names("m\xb6"),
     "m",
     info="ASCII that is not in the expected range without being replaceable is removed"
   )
-  
+
   # Fix issue #388 (that issue was specific to \xb2)
   # expect_equal(
   #   make_clean_names("m\x83\x84\x85\x86\x87\xa1"),
@@ -164,16 +164,18 @@ test_that("locale-specific make_clean_names tests", {
     "介護_看護_女",
     info="Unicode transliteration does not happen without make.names() and without ascii"
   )
-  expect_equal(
+  # warning due to unhandled mu
+  expect_warning(expect_equal(
     make_clean_names("μ"),
     "m",
     info="lower-case mu is transliterated to an 'm'"
-  )
-  expect_equal(
+  ))
+  # warning due to unhandled mu
+  expect_warning(expect_equal(
     make_clean_names("µ", ascii=FALSE, use_make_names=FALSE),
     "µ",
     info="lower-case mu is not transliterated to an 'm' and uses the "
-  )
+  ))
   Sys.setlocale(locale=orig_locale)
 })
 
@@ -184,6 +186,78 @@ test_that("do not create duplicates (fix #251)", {
   )
 })
 
+
+test_that("allow for duplicates (fix #495)", {
+  expect_equal(
+    make_clean_names(c("a", "a", "a_2"), allow_dupes = TRUE),
+    c("a", "a", "a_2")
+  )
+})
+
+test_that("warnings are issued when micro/mu are not handled (fix #448)", {
+  # Warning due to partially handled mu
+  expect_warning(expect_equal(
+    make_clean_names(string=c("\u00b5g", "\u03bcg"), replace=c("\u00b5g"="ug")),
+    c("ug", "mg")
+  ))
+  # The warning message should match the output of warn_micro_mu()
+  expect_equal(
+    tryCatch(
+      make_clean_names(string=c("\u00b5g", "\u03bcg"), replace=c("\u00b5g"="ug")),
+      warning=function(w) w$message
+    ),
+    tryCatch(
+      warn_micro_mu(string=c("\u00b5g", "\u03bcg"), replace=c("\u00b5g"="ug")),
+      warning=function(w) w$message
+    )
+  )
+})
+
+# Tests for warn_micro_mu ####
+
+test_that("warn_micro_mu", {
+  # No mu or micro generates no output and returns false
+  expect_false(
+    expect_silent(
+      warn_micro_mu(string=c("a"), replace=character())
+    )
+  )
+  # A micro or mu in the input without a replacement gives a warning and returns
+  # true
+  expect_warning(
+    expect_true(
+      warn_micro_mu(string=c("\u00b5g"), replace=character())
+    ),
+    regexp="The following characters are in the names to clean but are not replaced: \\u00b5",
+    fixed=TRUE
+  )
+  # A micro or mu in the input without a replacement gives a warning with all
+  # the characters and returns true
+  expect_warning(
+    expect_true(
+      warn_micro_mu(string=c("\u00b5g", "\u03bcg"), replace=character())
+    ),
+    regexp="The following characters are in the names to clean but are not replaced: \\u00b5, \\u03bc",
+    fixed=TRUE
+  )
+  # Multiple potential types of problems in a single message (split into two
+  # tests because multiline regular expressions are a pain).
+  expect_warning(
+    expect_true(
+      warn_micro_mu(string=c("\u00b5g", "\u03bcg"), replace=c("\u00b5g"="ug"))
+    ),
+    regexp="The following characters are in the names to clean but are not replaced: \\u03bc",
+    fixed=TRUE
+  )
+  expect_warning(
+    expect_true(
+      warn_micro_mu(string=c("\u00b5g", "\u03bcg"), replace=c("\u00b5g"="ug"))
+    ),
+    regexp="The following characters are in the names to clean but may not be replaced, check the output names carefully: \\u00b5",
+    fixed=TRUE
+  )
+})
+
 # Tests for clean_names ####
 testing_vector <-
   c(
@@ -211,17 +285,25 @@ testing_vector <-
     "µ_first_unicode"
   )
 
+# These are individually checked above.  Here create an "answer key" vector
+# for use below, e.g., in testing clean_names.tbl_lazy
+expect_warning(snake_case_vector <- make_clean_names(testing_vector))
+
 test_df <- as.data.frame(matrix(ncol = 22))
 names(test_df) <- testing_vector
-clean <- clean_names(test_df, "snake", ascii=TRUE)
-clean_noascii <- clean_names(test_df, "snake", ascii=FALSE)
-
-test_that("Returns a data.frame", {
-  expect_is(clean, "data.frame")
+test_that("clean_names returns a data.frame", {
+  expect_warning(
+    clean <- clean_names(test_df, "snake", ascii=TRUE)
+  )
+  expect_warning(
+    clean_noascii <- clean_names(test_df, "snake", ascii=FALSE)
+  )
+  expect_s3_class(clean, "data.frame")
 })
 
 test_that("Tests for cases beyond default snake", {
-  expect_equal(
+  # warning due to unhandled mu
+  expect_warning(expect_equal(
     names(clean_names(test_df, "small_camel")),
     c(
       "spAce", "repeated", "a", "percent", "x", "x_2", "d9", "repeated_2",
@@ -229,8 +311,9 @@ test_that("Tests for cases beyond default snake", {
       "testCamelCase", "leadingpunct", "averageNumberOfDays",
       "jan2009Sales", "jan2009Sales_2", "notFirstUnicodeM", "mFirstUnicode"
     )
-  )
-  expect_equal(
+  ))
+  # warning due to unhandled mu
+  expect_warning(expect_equal(
     names(clean_names(test_df, "big_camel")),
     c(
       "SpAce", "Repeated", "A", "Percent", "X", "X_2", "D9", "Repeated_2",
@@ -238,8 +321,9 @@ test_that("Tests for cases beyond default snake", {
       "Leadingpunct", "AverageNumberOfDays", "Jan2009Sales", "Jan2009Sales_2",
       "NotFirstUnicodeM", "MFirstUnicode"
     )
-  )
-  expect_equal(
+  ))
+  # warning due to unhandled mu
+  expect_warning(expect_equal(
     names(clean_names(test_df, "all_caps")),
     c(
       "SP_ACE", "REPEATED", "A", "PERCENT", "X", "X_2", "D_9", "REPEATED_2",
@@ -248,8 +332,9 @@ test_that("Tests for cases beyond default snake", {
       "JAN_2009_SALES",
       "NOT_FIRST_UNICODE_M", "M_FIRST_UNICODE"
     )
-  )
-  expect_equal(
+  ))
+  # warning due to unhandled mu
+  expect_warning(expect_equal(
     names(clean_names(test_df, "lower_upper")),
     c(
       "spACE", "repeated", "a", "percent", "x", "x_2", "d9", "repeated_2",
@@ -257,8 +342,9 @@ test_that("Tests for cases beyond default snake", {
       "testCAMELcase", "leadingpunct", "averageNUMBERofDAYS", "jan2009SALES",
       "jan2009SALES_2", "notFIRSTunicodeM", "mFIRSTunicode"
     )
-  )
-  expect_equal(
+  ))
+  # warning due to unhandled mu
+  expect_warning(expect_equal(
     names(clean_names(test_df, "upper_lower")),
     c(
       "SPace", "REPEATED", "A", "PERCENT", "X", "X_2", "D9", "REPEATED_2",
@@ -266,16 +352,17 @@ test_that("Tests for cases beyond default snake", {
       "TESTcamelCASE", "LEADINGPUNCT", "AVERAGEnumberOFdays", "JAN2009sales",
       "JAN2009sales_2", "NOTfirstUNICODEm", "MfirstUNICODE"
     )
-  )
-  expect_equal(
+  ))
+  # warning due to unhandled mu
+  expect_warning(expect_equal(
     names(clean_names(test_df, "none")),
     c(
       "sp_ace", "repeated", "a", "percent", "X", "X_2", "d_9", "REPEATED",
-      "cant", "hi_there", "leading_spaces", "X_3", "acao", "Faroe", "a_b_c_d_e_f", 
-      "testCamelCase", "leadingpunct", "average_number_of_days", 
+      "cant", "hi_there", "leading_spaces", "X_3", "acao", "Faroe", "a_b_c_d_e_f",
+      "testCamelCase", "leadingpunct", "average_number_of_days",
       "jan2009sales", "jan_2009_sales", "not_first_unicode_m", "m_first_unicode"
     )
-  )
+  ))
   expect_equal(
     names(clean_names(test_df, "old_janitor")),
     c(
@@ -286,55 +373,103 @@ test_that("Tests for cases beyond default snake", {
     )
   )
   # check that alias arguments yield identical outputs
-  expect_equal(names(clean_names(test_df, "screaming_snake")), names(clean_names(test_df, "all_caps")))
-  expect_equal(names(clean_names(test_df, "big_camel")), names(clean_names(test_df, "upper_camel")))
-  expect_equal(names(clean_names(test_df, "small_camel")), names(clean_names(test_df, "lower_camel")))
+  # warning due to unhandled mu
+  expect_warning(expect_warning(
+    expect_equal(
+      names(clean_names(test_df, "screaming_snake")),
+      names(clean_names(test_df, "all_caps"))
+    )
+  ))
+  # warning due to unhandled mu
+  expect_warning(expect_warning(
+    expect_equal(
+      names(clean_names(test_df, "big_camel")),
+      names(clean_names(test_df, "upper_camel"))
+    )
+  ))
+  # warning due to unhandled mu
+  expect_warning(expect_warning(
+    expect_equal(
+      names(clean_names(test_df, "small_camel")),
+      names(clean_names(test_df, "lower_camel"))
+    )
+  ))
 })
 
-test_that("errors if not called on a data.frame", {
+test_that("Tests for clean_names.default() on lists and vectors", {
+  test_v <- seq_along(testing_vector)
+  names(test_v) <- testing_vector
+  test_list <- as.list(test_v)
+
+  # Warnings due to partially handled mu
+  expect_warning(
+    clean_v <- clean_names(test_v),
+    regexp="mu or micro symbol"
+  )
+  expect_warning(
+    clean_l <- clean_names(test_list),
+    regexp="mu or micro symbol"
+  )
+  expect_equal(names(clean_v)[1], "sp_ace")
+  expect_equal(names(clean_l)[1], "sp_ace")
+  expect_type(clean_v, "integer")
+  expect_type(clean_l, "list")
+
+  unnamed <- seq_along(testing_vector)
   expect_error(
-    clean_names(1:3),
-    regexp="No `clean_names()` method exists for the class integer",
+    clean_names(unnamed),
+    regexp="requires that either names or dimnames be non-null.",
     fixed=TRUE
   )
 })
 
+test_that("Tests for clean_names.default() on arrays", {
+  x = array(NA, dim = c(2, 2, 2), dimnames = list(c("A", "B"), c("C", "D"), c("E", "F")))
+  clean = clean_names(x)
+  test = unlist(dimnames(x))
+  expect_false(isTRUE(all.equal(test, tolower(test))))
+  test_clean = unlist(dimnames(clean))
+  expect_true(isTRUE(all.equal(test_clean, tolower(test_clean))))
+})
 
-#------------------------------------------------------------------------------# 
+
+#------------------------------------------------------------------------------#
 #---------------------------- Tests for sf method -----------------------------####
 #------------------------------------------------------------------------------#
 
-context("clean_names.sf")
-
 test_that("Names are cleaned appropriately without attaching sf", {
   skip_if_not_installed("sf")
-  nc    <- sf::st_read(system.file("shape/nc.shp", package="sf"))
+  # capture.output is used to hide internal messages from the sf package on
+  # loading
+  capture.output(
+    nc <- sf::st_read(system.file("shape/nc.shp", package="sf"))
+  )
   clean <- clean_names(nc, "snake")
-  
+
   expect_equal(names(clean)[4], "cnty_id")
-  expect_is(clean, "sf")
+  expect_s3_class(clean, "sf")
 })
 
 test_that("Names are cleaned appropriately", {
   skip_if_not_installed("sf")
   test_df <- data.frame(matrix(ncol = 22) %>% as.data.frame())
-  
+
   names(test_df) <- c(
     "sp ace", "repeated", "a**^@", "%", "*", "!",
     "d(!)9", "REPEATED", "can\"'t", "hi_`there`", "  leading spaces",
     "€", "ação", "Farœ", "a b c d e f", "testCamelCase", "!leadingpunct",
     "average # of days", "jan2009sales", "jan 2009 sales", "long", "lat"
   )
-  
+
   test_df["long"] <- -80
   test_df["lat"] <- 40
-  
+
   test_df <- sf::st_as_sf(test_df, coords = c("long", "lat"))
   names(test_df)[21] <- "Geometry"
   sf::st_geometry(test_df) <- "Geometry"
-  
+
   clean <- clean_names(test_df, "snake")
-  
+
   expect_equal(names(clean)[1], "sp_ace") # spaces
   expect_equal(names(clean)[2], "repeated") # first instance of repeat
   expect_equal(names(clean)[3], "a") # multiple special chars, trailing special chars
@@ -361,21 +496,21 @@ test_that("Names are cleaned appropriately", {
 test_that("Tests for cases beyond default snake for sf objects", {
   skip_if_not_installed("sf")
   test_df <- data.frame(matrix(ncol = 22) %>% as.data.frame())
-  
+
   names(test_df) <- c(
     "sp ace", "repeated", "a**^@", "%", "*", "!",
     "d(!)9", "REPEATED", "can\"'t", "hi_`there`", "  leading spaces",
     "€", "ação", "Farœ", "a b c d e f", "testCamelCase", "!leadingpunct",
     "average # of days", "jan2009sales", "jan 2009 sales", "long", "lat"
   )
-  
+
   test_df["long"] <- -80
   test_df["lat"] <- 40
-  
+
   test_df <- sf::st_as_sf(test_df, coords = c("long", "lat"))
   names(test_df)[21] <- "geometry"
   sf::st_geometry(test_df) <- "geometry"
-  
+
   expect_equal(
     names(clean_names(test_df, "small_camel")),
     c(
@@ -415,8 +550,8 @@ test_that("Tests for cases beyond default snake for sf objects", {
     names(clean_names(test_df, "none")),
     c(
       "sp_ace", "repeated", "a", "percent", "X", "X_2", "d_9", "REPEATED",
-      "cant", "hi_there", "leading_spaces", "X_3", "acao", "Faroe", "a_b_c_d_e_f", 
-      "testCamelCase", "leadingpunct", "average_number_of_days", 
+      "cant", "hi_there", "leading_spaces", "X_3", "acao", "Faroe", "a_b_c_d_e_f",
+      "testCamelCase", "leadingpunct", "average_number_of_days",
       "jan2009sales", "jan_2009_sales", "geometry"
     )
   )
@@ -430,30 +565,26 @@ test_that("Tests for cases beyond default snake for sf objects", {
   )
 })
 
-#------------------------------------------------------------------------------# 
+#------------------------------------------------------------------------------#
 #------------------------ Tests for tbl_graph method --------------------------#####
 #------------------------------------------------------------------------------#
 
-context("clean_names.tbl_graph")
-
 test_that("tbl_graph/tidygraph", {
   skip_if_not_installed("tidygraph")
   # create test graph to test clean_names
   test_graph <-
-    tidygraph::play_erdos_renyi(10, 0.5) %>% 
+    tidygraph::play_erdos_renyi(10, 0.5) %>%
     # create nodes wi
-    tidygraph::bind_nodes(test_df) %>% 
+    tidygraph::bind_nodes(test_df) %>%
     tidygraph::mutate_all(tidyr::replace_na, 1)
 
   # create a graph with clean names
-  clean_graph <- clean_names(test_graph, case = "snake")
+  # warning due to unhandled mu
+  expect_warning(clean_graph <- clean_names(test_graph, case = "snake"))
 
   # get clean names
   clean <- names(tibble::as_tibble(clean_graph))
-  expect_is(
-    clean_graph, "tbl_graph",
-    info="Returns a tbl_graph object"
-  )
+  expect_s3_class(clean_graph, "tbl_graph")
 
   expect_equal(clean[1], "sp_ace") # spaces
   expect_equal(clean[2], "repeated") # first instance of repeat
@@ -478,6 +609,37 @@ test_that("tbl_graph/tidygraph", {
   expect_equal(clean[20], "jan_2009_sales") # yes separator around number-word boundary if it existed
 })
 
+
+#-----------------------------------------------------------------------------#
+#------------------------ Tests for tbl_lazy method --------------------------#####
+#-----------------------------------------------------------------------------#
+
+
+test_that("tbl_lazy/dbplyr", {
+  skip_if_not_installed("dbplyr")
+  skip_if_not_installed("RSQLite")
+    # can't have column names "*" or a true repeat in the db names.  Work around this by
+    # switching in other column names whose output will match the testing vector
+    test_db <- dbplyr::memdb_frame(test_df %>%
+                                     dplyr::select(-"*", -REPEATED)) %>% # these two cases break the db
+      dplyr::mutate(repeated_2 = repeated, x = NA) %>%
+      dplyr::select(c(testing_vector[1:4],
+               "x",
+               testing_vector[6:7],
+               "repeated_2",
+               testing_vector[9:22]))
+    
+    # create a database object with clean names
+    # warning due to unhandled mu
+    expect_warning(clean_db <- clean_names(test_db, case = "snake"))
+    clean_db_names <- colnames(clean_db)
+    expect_equal(
+      clean_db_names,
+      snake_case_vector
+    )
+})
+
+
 test_that("Work around incomplete stringi transliterators (Fix #365)", {
   options(janitor_warn_transliterators=NULL)
   expect_warning(
@@ -497,6 +659,6 @@ test_that("Work around incomplete stringi transliterators (Fix #365)", {
 test_that("groupings are preserved, #260", {
   df_grouped <- iris %>% dplyr::group_by(Sepal.Length, Sepal.Width) # nonsense for analysis but doesn't matter
   df_grouped_renamed <- df_grouped %>% clean_names(case = "lower_camel")
-  expect_equal(group_vars(df_grouped_renamed), c("sepalLength", "sepalWidth")) # group got renamed
+  expect_equal(dplyr::group_vars(df_grouped_renamed), c("sepalLength", "sepalWidth")) # group got renamed
   expect_equal(names(df_grouped_renamed), c("sepalLength", "sepalWidth", "petalLength", "petalWidth", "species"))
 })
